/* lif_block.c -- read a block (specified by logical block number from
                  a lif disk or image file */ 
/*  2000 A. R. Duell and placed under the GPL */

#include<stdio.h>
#include<unistd.h>
#include<fcntl.h>
#include"lif_phy.h"

#define SECTOR_SIZE 256

static int current_cylinder;

void lif_init(int input_device)
  {
    /* Move the drive head to cylinder 0 and set current_cylinder */
    lif_recalibrate(input_device);
    current_cylinder=0;
  }

void lif_read_phy_block(int input_device, int block, unsigned char *data)
  {
    /* Read one block from a physical LIF disk */
    int cylinder, head, sector;

    /* Calculate where this block is on the disk */
    cylinder=block/32;
    head=(block/16)%2;
    sector=(block%16)+1;

    /* If we're not on the right cylinder, go there */
    if(cylinder!=current_cylinder)
      {
        lif_seek(input_device,cylinder);
        current_cylinder=cylinder;
      }

    /* Now read the block */
    lif_read(input_device,cylinder,head,sector,data);
  }

void lif_read_img_block(int input_file, int block, unsigned char *data)
  {
    /* Read one block from an image file */

    /* Go to the right block in the file */
    lseek(input_file,(SECTOR_SIZE*block),SEEK_SET);
    /* Read it */
    if(read(input_file,data,SECTOR_SIZE)!=SECTOR_SIZE)
      {
        fprintf(stderr,"Error reading block %d from file\n",block);
        exit(1);
      }
  }

void lif_read_block(int input_device, int file_flag, int block, 
                    unsigned char *data)
  {
    /* Read one block */
    if (file_flag)
      {
        lif_read_img_block(input_device,block,data);
      }
    else
      {
        lif_read_phy_block(input_device,block,data);
      }
  }
